//    IPS Java Speech Database
//    (c) Copyright 2011-2020
//    Institute of Phonetics and Speech Processing,
//    Ludwig-Maximilians-University, Munich, Germany
//
//
//    This file is part of IPS Java Speech Database
//
//
//    IPS Java Speech Database is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General Public License as published by
//    the Free Software Foundation, version 3 of the License.
//
//    IPS Java Speech Database is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with IPS Java Speech Database.  If not, see <http://www.gnu.org/licenses/>.

package ipsk.apps.speechrecorder.db;



import java.text.DateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import javax.xml.bind.annotation.XmlType;

import ipsk.beans.PreferredDisplayOrder;
import ipsk.beans.dom.DOMAttributes;
import ipsk.beans.dom.DOMElements;
import ipsk.db.speech.InformedConsent;


@DOMAttributes("personId")
@DOMElements({"name","forename","address","sex","dateOfBirth","birthPlace","profession","dialectRegion","additionalLanguage","comments"})
@PreferredDisplayOrder("name,forename,address,sex,dateOfBirth,height,weight,smoker,brace,mouthPiercing,birthPlace,zipcode,profession,dialectRegion,motherTongue,motherTongueMother,motherTongueFather,additionalLanguage,comments")
//@XmlType(name="speaker",propOrder={"name","forename","address","sex","dateOfBirth","height","weight","smoker","brace","mouthPiercing","birthPlace","zipcode","profession","dialectRegion","motherTongue","motherTongueMother","motherTongueFather","additionalLanguage","comments"})
@XmlType(name="speaker",namespace="speechrecorder")
public class Speaker extends ipsk.db.speech.Speaker{
	public final static String ELEMENT_NAME="speaker";

	
	// backward compatibility to Speechrecorder speakers.txt file
    private String gender;
    
    // backward compatibility to Speechrecorder speakers.txt file
    private String dateOfBirthString;
    
    private DateFormat dateOfBirthFormat=DateFormat.getDateInstance();
    
	public Speaker() {
		super();
		
	}

	public Speaker(int personId) {
	    super(personId);
	    
	    // Bugfix:ID0120 2023-11-16
	    //"Speaker XML database file contains empty <informedConsents/> element."
	    //Desktop SpeechRecorder does not provide informed consent forms. The element should not be there.
	    //Fix set informed consents to null for new speaker objects. Existing speaker entries will not be changed.
	    setInformedConsents(null);
	
	}

    public String getGender() {
        Sex s=getSex();
        if(s!=null){
            return s.toString();
        }
        return gender;
    }

    public void setGender(String gender) {
        this.gender = gender;
    }

    public String getDateOfBirthString() {
        Date dob=getDateOfBirth();
        if(dob!=null){
            return dateOfBirthFormat.format(dob);
        }
        return dateOfBirthString;
    }

    public void setDateOfBirthString(String dateOfBirthString) {
        this.dateOfBirthString = dateOfBirthString;
    }

   

}
